MODULE UnixPlot;

 (*
    UnixPlot - Plotting library interface.       
    Copyright (C) 1996 Michael Griebling
 
    This module is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This module is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*)

IMPORT pl:= UnixPlotLibrary,
       m:= LRealMath, Str:= Strings;

CONST
  XDIV = 4;
  YDIV = 4;

VAR
  func : pl.PlotFunction;
  Plot : pl.PlotType;
  xDivs - : INTEGER;
  yDivs - : INTEGER;
  mode - : INTEGER;
  xDec - : INTEGER;
  yDec - : INTEGER;
  xMin -, xMax -, yMin -, yMax - : LONGREAL;


PROCEDURE DetermineDecimals (VAR str : ARRAY OF CHAR; VAR dec : INTEGER);
VAR pos, olddec : INTEGER; found: BOOLEAN;
BEGIN
  olddec := dec; dec := 0;
  Str.FindNext(".", str, 0, found, pos); INC(pos);
  WHILE (pos > 0) & (str[pos] # 0X) & (str[pos] # "E") DO
    INC(pos); INC(dec)
  END;
  IF str[pos] = "E" THEN dec := 0
  ELSIF olddec > dec THEN dec := olddec
  END
END DetermineDecimals;

PROCEDURE SetMode (arg : LONGINT);
BEGIN
  mode := SHORT(arg);
  (*
  gt.SetGadgetAttrs(p.PlotGadgets[p.GDTypeField]^, p.PlotWnd, NIL,
                    gt.cyActive, arg);
  *)
END SetMode;

PROCEDURE SetDecimalsAndMinMax;
VAR (* si : I.StringInfoPtr; *)
  ok : BOOLEAN;
  buffer : ARRAY 80 OF CHAR;
BEGIN
  SetMode(mode);
  ok := pl.RealToString(xMin, buffer, 1, xDec, " ");
  (*
  gt.SetGadgetAttrs(p.PlotGadgets[p.GDXMinField]^, p.PlotWnd, NIL,
                    gt.stString, sys.ADR(buffer));
  ok := pl.RealToString(xMax, buffer, 1, xDec, " ");
  gt.SetGadgetAttrs(p.PlotGadgets[p.GDXMaxField]^, p.PlotWnd, NIL,
                    gt.stString, sys.ADR(buffer));
  ok := pl.RealToString(yMin, buffer, 1, yDec, " ");
  gt.SetGadgetAttrs(p.PlotGadgets[p.GDYMinField]^, p.PlotWnd, NIL,
                    gt.stString, sys.ADR(buffer));
  ok := pl.RealToString(yMax, buffer, 1, yDec, " ");
  gt.SetGadgetAttrs(p.PlotGadgets[p.GDYMaxField]^, p.PlotWnd, NIL,
                    gt.stString, sys.ADR(buffer));
  gt.SetGadgetAttrs(p.PlotGadgets[p.GDXDivSlider]^, p.PlotWnd, NIL,
                    gt.slLevel, I.LongToUInt(xDivs));
  gt.SetGadgetAttrs(p.PlotGadgets[p.GDYDivSlider]^, p.PlotWnd, NIL,
                    gt.slLevel, I.LongToUInt(yDivs));
  *)
END SetDecimalsAndMinMax;

PROCEDURE GetDecimalsAndMinMax;
VAR (* si : I.StringInfoPtr; *)
  ok : BOOLEAN;
BEGIN
  xDec := 0; yDec := 0;
  (*
  si := p.PlotGadgets[p.GDXMinField].specialInfo;
  ok := rc.StringToReal(si.buffer^, xMin);
  DetermineDecimals(si.buffer^, xDec);
  si := p.PlotGadgets[p.GDXMaxField].specialInfo;
  ok := rc.StringToReal(si.buffer^, xMax);
  DetermineDecimals(si.buffer^, xDec);
  si := p.PlotGadgets[p.GDYMinField].specialInfo;
  ok := rc.StringToReal(si.buffer^, yMin);
  DetermineDecimals(si.buffer^, yDec);
  si := p.PlotGadgets[p.GDYMaxField].specialInfo;
  ok := rc.StringToReal(si.buffer^, yMax);
  DetermineDecimals(si.buffer^, yDec)
  *)
END GetDecimalsAndMinMax;

PROCEDURE SetState * (m, xDc, yDc : INTEGER;
                      xMn, xMx, yMn, yMx : LONGREAL;
                      xD, yD : INTEGER);
BEGIN
  mode := m; xDivs := xD; yDivs := yD; xDec := xDc; yDec := yDc;
  xMin := xMn; xMax := xMx; yMin := yMn; yMax := yMx;
END SetState;

PROCEDURE HandlePlot () : BOOLEAN;
VAR
  (*
  imsgptr: I.IntuiMessagePtr;
  imsg: I.IntuiMessage;
  item: I.MenuItemPtr;
  *)
  Code: INTEGER;
  done: BOOLEAN;
  (* gad : I.GadgetPtr; *)

  PROCEDURE UpdateYStrings;
  BEGIN
    GetDecimalsAndMinMax;
    pl.SetPlotLimits(Plot, xMin, xMax, yMin, yMax);
    pl.NewYLimits(Plot, func);
    yMin := Plot.Miny; yMax := Plot.Maxy;
    SetDecimalsAndMinMax;
  END UpdateYStrings;

  PROCEDURE UpdatePlot;
  BEGIN
    (* initialize the decimals and min/max values *)
    GetDecimalsAndMinMax;

    (* test if this is a log mode & check for negative/zero limits *)
    IF ODD(mode) THEN (* check x limits *)
      IF (xMin <= 0.0) OR (xMax <= 0.0) THEN SetMode(pl.Normal) END;
    END;
    IF mode >= pl.LogY THEN (* check y limits *)
      IF (yMin <= 0.0) OR (yMax <= 0.0) THEN SetMode(pl.Normal) END;
    END;

    (* draw a new plot *)
    pl.DonePlot(Plot);   (* clear out old plot *)
    IF pl.InitPlot(Plot, SHORT(mode),
                   xMin, xMax, yMin, yMax, xDivs, yDivs,
                   5, 5, xDec, yDec) THEN
      pl.PlotFx(Plot, func)
    END
  END UpdatePlot;

BEGIN
  done := FALSE;
  (*
  LOOP
    imsgptr := gt.GetIMsg (p.PlotWnd.userPort);
    IF imsgptr = NIL THEN EXIT END;
    imsg := imsgptr^;                       (* copy message *)
    gt.ReplyIMsg (imsgptr); imsgptr := NIL; (* return original *)

    IF I.closeWindow IN imsg.class THEN
      done := TRUE;
    ELSIF I.gadgetUp IN imsg.class THEN
      gad := imsg.iAddress;
      IF gad = p.PlotGadgets[p.GDPlotButton] THEN UpdatePlot
      ELSIF gad = p.PlotGadgets[p.GDNewYButton] THEN UpdateYStrings
      ELSIF gad = p.PlotGadgets[p.GDTypeField] THEN mode := imsg.code
      ELSIF gad = p.PlotGadgets[p.GDXDivSlider] THEN xDivs := imsg.code
      ELSIF gad = p.PlotGadgets[p.GDYDivSlider] THEN yDivs := imsg.code
      ELSE (* ignore string gadgets *)
      END
    ELSIF (I.refreshWindow IN imsg.class) THEN
      gt.BeginRefresh (p.PlotWnd);
      p.PlotRender;
      IF ~(I.zoomed IN p.PlotWnd.flags) THEN
        pl.PlotFx(Plot, func)
      END;
      gt.EndRefresh (p.PlotWnd, I.LTRUE);
    END;
  END; (* LOOP *)
  *)
  RETURN done;
END HandlePlot;


PROCEDURE PlotFunction * (f : pl.PlotFunction);
VAR
  quit : BOOLEAN;
BEGIN
  (* open everything *)
  quit := FALSE;
  (*
  rt.Assert(p.SetupScreen() = 0, "Unable to open screen!");
  rt.Assert(p.OpenPlotWindow(TRUE, TRUE) = 0, "Unable to open window!");
  SetDecimalsAndMinMax;
  rt.Assert(pl.InitPlot(Plot, p.PlotWnd, SHORT(mode),
                        xMin, xMax, yMin, yMax,
                        xDivs, yDivs, 5, 5, xDec, yDec),
                        "Unable to initialize plot!");

  (* perform a simple plot *)
  func := f; pl.PlotFx(Plot, func);

  (* handle any signals from the user *)
  REPEAT
    quit := (d.ctrlC IN e.Wait(LONGSET{p.PlotWnd.userPort.sigBit, d.ctrlC}));
    quit := quit OR HandlePlot()
  UNTIL quit;
  *)

  (* exit and shut everything down *)
  pl.DonePlot(Plot);
  (*
  p.ClosePlotWindow;
  p.CloseDownScreen
  *)
END PlotFunction;


PROCEDURE Sin (x : LONGREAL) : LONGREAL;
BEGIN
  RETURN m.sin(x)
END Sin;


BEGIN
  func := Sin;
  Plot := NIL;
  xDivs := XDIV; yDivs := YDIV;
  mode := pl.Normal;
  xDec := 0;
  yDec := 1;
  xMin := 0; xMax := 360; yMin := -1; yMax := 1;
END UnixPlot.
